SystemOrganization addCategory: #'AlgebraicDataType-Tests'!
SystemOrganization addCategory: #AlgebraicDataType!
SystemOrganization addCategory: #'AlgebraicDataType-Trees'!

!Collection methodsFor: '*AlgebraicDataType' stamp: 'fbs 7/9/2011 16:02'!
join: delimiter
	^ String streamContents: [ :stream |
		self
			do: [ :each | each putOn: stream ]
			separatedBy: [ delimiter putOn: stream ] ]! !

Object subclass: #AlgebraicDataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!AlgebraicDataType commentStamp: 'fbs 5/25/2011 07:30' prior: 0!
My subclasses represent algebraic types. My subclasses know how to destructure themselves through implementing #unapply.

Further, for unification between two graphs of instances of my subclasses, you may think of the instances as functions. For instance, Node left: a right: b is conceptually the same as some function Node(left, right) taking Trees as parameters. Thus, you may access a structure's parts through a parameter-like API in #paramAt:, #numArgs, etc.!

!AlgebraicDataType class methodsFor: 'private' stamp: 'fbs 7/9/2011 16:55'!
accessorFor: selector
	selector numArgs > 0 ifTrue: [Error signal: 'Accessor must have arity 0'].
	
	^ {selector asString. '	^ ', selector asString , '.'} join: Character cr.! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:31'!
apply: aCollection
	"The inverse operation to #unapply: given a tagged collection, create an object."
	self subclassResponsibility.! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:47'!
assertCorrectSize: aCollection
	| expected |
	expected := self new unapply size.
	(aCollection size = expected)
		ifFalse: [ApplyError signal: ('Incorrect number of arguments: was {1}, expected {2}' format: {aCollection size. expected})].! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:46'!
assertCorrectTag: aCollection
	| expected |
	expected := self new unapply first.
	(aCollection first = expected)
		ifFalse: [ApplyError signal: ('Unexpected tag: was {1}, expected {2}' format: {aCollection first. expected})].! !

!AlgebraicDataType class methodsFor: 'private' stamp: 'fbs 7/9/2011 15:07'!
constructorFor: selector
	| tokens interleavedNames |
	selector numArgs = 0 ifTrue: [Error signal: 'Selector must have arity > 0'].
	tokens := selector allButLast splitBy: #:.
	interleavedNames := (tokens collect: [:each | each, ': ', each]) reduce: [:acc :each | acc, ' ', each].
	^ interleavedNames, Character cr,
		'	^ self new ', interleavedNames, '.'! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/9/2011 17:20'!
define: className in: category
	"Define the root class of an algebraic data type.
	- Return the new class."
	^ AlgebraicDataType
		subclass: className
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: category.! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/10/2011 08:55'!
define: className in: category ofType: superclass withConstructor: selector
	"Define a new class with superclass superclass.
	- Define a class-side constructor called selector. This calls out to a private instance-side initialiser.
	- Define instvars by deconstructing selector - #a:b:c: will define instvars 'a b c'.
	- Define accessors for those instvars.
	- Return the new class."
	| newClass varNames |
	selector numArgs = 0 ifTrue: [Error signal: 'Selector must have arity > 0'].
	varNames := selector allButLast splitBy: #:.
	newClass := superclass
		subclass: className
		instanceVariableNames: (varNames reduce: [:acc :each | acc, ' ', each asString])
		classVariableNames: ''
		poolDictionaries: ''
		category: category.
	"Type constructor"
	newClass class compile: (self constructorFor: selector).
	newClass class organization addCategory: 'instance creation'.
	newClass class organization classify: selector under: 'instance creation'.
	"Accessors"
	newClass organization addCategory: 'accessing'.
	varNames do: [:each |
		newClass compile: (self accessorFor: each).
		newClass organization classify: each under: 'accessing'].
	"Initialiser"
	newClass organization addCategory: 'private'.
	newClass compile: (self initialiserFor: selector).
	newClass organization classify: selector under: 'private'.
	^ newClass.! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/9/2011 17:09'!
define: className in: category withConstructor: selector
	^ self
		define: className
		in: category
		ofType: AlgebraicDataType
		withConstructor: selector.! !

!AlgebraicDataType class methodsFor: 'instance creation' stamp: 'fbs 7/9/2011 14:32'!
define: className withConstructor: selector
	^ self
		define: className
		in: AlgebraicDataType category
		ofType: AlgebraicDataType
		withConstructor: selector.! !

!AlgebraicDataType class methodsFor: 'private' stamp: 'fbs 7/9/2011 16:03'!
initialiserFor: selector
	| tokens names |
	selector numArgs = 0 ifTrue: [Error signal: 'Selector must have arity > 0'].
	
	tokens := selector allButLast splitBy: #:.
	names := tokens collect: [:each | each -> ('a', each capitalized)].

	^ ({(names collect: [:each | each key asString, ': ', each value]) join: ' '} ,
		(names collect: [:each | ('	{1} := {2}.' format: {each key. each value.})]))
			join: Character cr.! !

!AlgebraicDataType methodsFor: 'comparing' stamp: 'fbs 5/22/2011 21:14'!
= anObject
	^ (anObject respondsTo: #unapply)
		ifTrue: [self unapply = anObject unapply]
		ifFalse: [anObject isCollection
			ifTrue: [self unapply = anObject]
			ifFalse: [false]]! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/27/2011 22:06'!
arity
	^ self children size.! !

!AlgebraicDataType methodsFor: 'accessing' stamp: 'fbs 5/24/2011 22:06'!
children
	^ self unapply allButFirst.! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/30/2011 13:28'!
hasChildren
	"Return true if self contains substructures/is of the form f(x1, x2, ..., xn)."
	"This covers most bases, but see MetaVariable for an exception."
	^ self children isEmpty not.! !

!AlgebraicDataType methodsFor: 'comparing' stamp: 'fbs 5/25/2011 22:54'!
hash
	"17's a prime."
	^ self unapply inject: 17 into: [:acc :each | 17 * each hash].! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:34'!
isAlgebraic
	^ true.! !

!AlgebraicDataType methodsFor: 'printing' stamp: 'fbs 7/23/2011 10:52'!
printOn: aStream
	aStream nextPut: $(.
	aStream nextPutAll: ((self unapply collect: [:token | token printString])
		reduce: [:acc :each | acc , ' ', each]).
	aStream nextPut: $).! !

!AlgebraicDataType methodsFor: 'accessing' stamp: 'fbs 5/25/2011 23:06'!
type
	^ self unapply first.! !

!AlgebraicDataType methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 20:29'!
unapply
	self subclassResponsibility.! !

!AlgebraicDataType methodsFor: 'destructuring' stamp: 'fbs 7/23/2011 10:55'!
unapplyFully
	^ self unapply collect: [:token | token unapplyFully]! !

AlgebraicDataType subclass: #Tree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree subclass: #Empty
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Empty class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:48'!
apply: aCollection
	self assertCorrectSize: aCollection.
	self assertCorrectTag: aCollection.
	^ self new.! !

!Empty methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:05'!
unapply
	^ #(#Empty).! !

Tree subclass: #Leaf
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Leaf class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:48'!
apply: aCollection
	self assertCorrectSize: aCollection.
	self assertCorrectTag: aCollection.
	^ self value: aCollection second.! !

!Leaf class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	^ self new value: anObject.! !

!Leaf methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:16'!
unapply
	^ {#Leaf. value.}! !

!Leaf methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:20'!
value
	^ value.! !

!Leaf methodsFor: 'private' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	value := anObject.! !

Tree subclass: #Node
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Node class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 17:56'!
apply: aCollection
	self assertCorrectSize: aCollection.
	self assertCorrectTag: aCollection.
	^ self left: (Tree apply: aCollection second) right: (Tree apply: aCollection third).! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject 
	^ self left: aLeftObject right: Tree empty.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject right: aRightObject
	^ self new left: aLeftObject right: aRightObject.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
right: aRightObject
	^ self left: Tree empty right: aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
left
	^ left.! !

!Node methodsFor: 'private' stamp: 'fbs 5/22/2011 20:22'!
left: aLeftObject right: aRightObject
	left := aLeftObject.
	right := aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
right
	^ right.! !

!Node methodsFor: 'destructuring' stamp: 'fbs 7/22/2011 18:25'!
unapply
	^ {#Node. left. right.}! !

!Tree class methodsFor: 'instance creation' stamp: 'fbs 7/22/2011 18:00'!
apply: aCollection
	| knownTags tag |
	aCollection isEmpty ifTrue: [ApplyError signal: 'Cannot apply the empty list'].
	tag := aCollection first.
	tag = #Empty ifTrue: [^ Empty apply: aCollection].
	tag = #Leaf ifTrue: [^ Leaf apply: aCollection].
	tag = #Node ifTrue: [^ Node apply: aCollection].
	knownTags := 'Empty, Leaf, Node'.
	ApplyError signal: ('Unknown tag {1}, expected one of {2}' format: {tag. knownTags}). ! !

!Tree class methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:27'!
empty
	^ EmptyTree.! !

!Tree class methodsFor: 'class initialization' stamp: 'fbs 5/22/2011 20:28'!
initialize
	EmptyTree := Empty new.! !

!Object methodsFor: '*AlgebraicDataType' stamp: 'fbs 5/25/2011 22:34'!
isAlgebraic
	^ false.! !

!Object methodsFor: '*AlgebraicDataType' stamp: 'fbs 7/23/2011 10:56'!
unapply
	^ self.! !

!Object methodsFor: '*AlgebraicDataType' stamp: 'fbs 7/23/2011 10:56'!
unapplyFully
	^ self.! !

TestCase subclass: #AlgebraicDataTypeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 17:01'!
testAccessorForNonZeroAritySelectorFails
	self should: [AlgebraicDataType accessorFor: #foo:] raise: Error.
	self should: [AlgebraicDataType accessorFor: #foo:bar:] raise: Error.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 16:56'!
testAccessorForNullarySelectorReturnsMethodString
	| src |
	src := 'foo
	^ foo.'.
	self assert: (AlgebraicDataType accessorFor: #foo) = src.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 15:40'!
testConstructorForNullarySelectorFails
	self should: [AlgebraicDataType constructorFor: #foo] raise: Error.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 14:55'!
testConstructorForReturnsMethodString
	self assert: (AlgebraicDataType constructorFor: #foo:) = 'foo: foo\	^ self new foo: foo.' withCRs.
	self assert: (AlgebraicDataType constructorFor: #foo:bar:) = 'foo: foo bar: bar\	^ self new foo: foo bar: bar.' withCRs.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 15:39'!
testInitialiserForHighAritySelectorInitialisesAllInstVars
	| src |
	src := 'foo: aFoo bar: aBar baz: aBaz
	foo := aFoo.
	bar := aBar.
	baz := aBaz.'.
	self assert: (AlgebraicDataType initialiserFor: #foo:bar:baz:) = src.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 15:15'!
testInitialiserForNullarySelectorFails
	self should: [AlgebraicDataType initialiserFor: #foo] raise: Error.! !

!AlgebraicDataTypeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/9/2011 15:39'!
testInitialiserForUnarySelectorInitialisesInstVar
	| src |
	src := 'foo: aFoo
	foo := aFoo.'.
	self assert: (AlgebraicDataType initialiserFor: #foo:) = src.! !

TestCase subclass: #EmptyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!EmptyTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:35'!
testApplyIsInverseOfUnapply
	| e |
	e := Empty new.
	self assert: e = (Empty apply: e unapply)! !

!EmptyTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:32'!
testEmptyListFailsToApply
	self should: [Empty apply: {}] raise: ApplyError.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 07:38'!
testEmptyTreeHasNoChildren
	self deny: Empty new hasChildren.
	self deny: Tree empty hasChildren.
	
	self assert: Tree empty children isEmpty.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:33'!
testEmptyTreeIsNotVariable
	self deny: Empty new isMetaVariable.
	self deny: Tree empty isMetaVariable.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:09'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Empty new.
	t2 := Empty new.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!EmptyTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:33'!
testIncorrectlyTaggedListFailsToApply
	self should: [Empty apply: {#empty}] raise: ApplyError.! !

!EmptyTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:34'!
testProperlyFormedListApplies
	self assert: ((Empty apply: {#Empty}) isKindOf: Empty)! !

!EmptyTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:33'!
testTwoElementListFailsToApply
	self should: [Empty apply: {#Empty. 1.}] raise: ApplyError.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:27'!
testUnapplyShouldDestructure
	self assert: Tree empty unapply = {#Empty}.! !

TestCase subclass: #LeafTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!LeafTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:41'!
testApplyIsInverseOfUnapply
	| leaf |
	leaf := Leaf value: 1.
	self assert: leaf = (Leaf apply: leaf unapply)! !

!LeafTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:38'!
testEmptyListFailsToApply
	self should: [Leaf apply: {}] raise: ApplyError.! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Leaf value: 1.
	t2 := Leaf value: 1.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 22:07'!
testLeafHasChildren
	self assert: (Leaf value: 1) hasChildren.
	
	self assert: (Leaf value: 1) children = #(1).! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:32'!
testLeafIsNotVariable
	self deny: (Leaf value: 1) isMetaVariable.! !

!LeafTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:38'!
testNoValueListFailsToApply
	self should: [Leaf apply: {#Leaf.}] raise: ApplyError.! !

!LeafTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:40'!
testProperlyFormedListApplies
	| leaf |
	leaf := Leaf apply: {#Leaf. 1}.
	
	self assert: (leaf isKindOf: Leaf).
	self assert: leaf value = 1.! !

!LeafTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:38'!
testTooLongListFailsToApply
	self should: [Leaf apply: {#Leaf. 1. 2.}] raise: ApplyError.! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testUnapplyShouldDestructure
	| destructure tree |
	tree := Leaf value: 1.
	destructure := tree unapply.
	self assert: destructure first = #Leaf.
	self assert: destructure second = tree value.! !

TestCase subclass: #NodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/23/2011 10:53'!
testApplyIsInverseOfUnapply
	| node |
	node := Node left: (Leaf value: 1) right: (Leaf value: 2).
	self assert: node = (Node apply: node unapplyFully).! !

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:50'!
testEmptyListFailsToApply
	self should: [Node apply: {}] raise: ApplyError.! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 20:58'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Node left: Tree empty.
	t2 := Node left: Tree empty.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 22:08'!
testNodeHasChildren
	self assert: (Node left: Tree empty right: Tree empty) hasChildren.
	
	self assert: (Node left: (Leaf value: 1) right: (Leaf value: 2)) children = {Leaf value: 1. Leaf value: 2}.! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:32'!
testNodeIsNotVariable
	self deny: (Node left: Tree empty right: Tree empty) isMetaVariable.! !

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/23/2011 10:44'!
testProperListApplies
	| node |
	node := Node apply: {#Node. {#Empty}. {#Leaf. 1.}}.
	self assert: (node isKindOf: Node).
	self assert: (node left isKindOf: Empty).
	self assert: node right value = 1.! !

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:55'!
testTooIncorrectlyTaggedListFailsToApply
	self should: [Node apply: {#Nod. {#Empty}. {#Empty}}] raise: ApplyError.! !

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:52'!
testTooLongListFailsToApply
	self should: [Node apply: {#Tree. {#Empty}. {#Empty}. {#Empty}}] raise: ApplyError.! !

!NodeTest methodsFor: 'testing - instance creation' stamp: 'fbs 7/22/2011 17:52'!
testTooShortListFailsToApply
	self should: [Node apply: {#Tree.}] raise: ApplyError.! !

!NodeTest methodsFor: 'testing - unapply' stamp: 'fbs 7/23/2011 10:56'!
testUnapplyFullyShouldDestructureEagerly
	| destructure tree |
	tree := Node left: (Leaf value: 1) right: (Leaf value: 2).
	destructure := tree unapplyFully.
	self assert: destructure first = #Node.
	self assert: destructure second isCollection.
	self assert: destructure second first = #Leaf.
	self assert: destructure third isCollection.
	self assert: destructure third first = #Leaf.! !

!NodeTest methodsFor: 'testing - unapply' stamp: 'fbs 5/22/2011 20:59'!
testUnapplyShouldDestructureLazily
	| destructure tree |
	tree := Node left: (Leaf value: 1) right: (Leaf value: 2).
	destructure := tree unapply.
	self assert: destructure first = #Node.
	self assert: destructure second = tree left.
	self assert: destructure third = tree right.! !

Error subclass: #ApplyError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!
Tree initialize!
